'Name: InstallHyperV.vbs
'Description: This script guides you through each step that is needed during the installation of Microsoft HyperV Core
'Author: Paul Freudenberger
'Licence: MIT License (MIT)
'Copyright (c) 2016 Paul Freudenberger
'Version: 2.2
'Version-Log:
'v2.2
'-Replaced powershell code with native VBS code if possible
'v2.1
'-Fixed a bug where SetKeyboardLayout always sets the same layout, no matter what choice the user has made
'v2.0
'-Public Release at paulfreudenberger.ddns.net
'v1.9
'-Extended the commented out functions by ShowScheduledTasks
'v1.8
'-Added a few functions that are commented out by default (Can be usefull if you might not know how to do certain things)
'v1.7
'-Added Install HVMTools because its a good freeware alternative to manage HyperV Core
'v1.6
'-Added WindowsBackupRole Installation because wbadmin.exe is not understandable in the first place
'-Added ConfigureDailyBackup so the hard part of planing a Backup&Restore-Concept is already done for you
'v1.5
'-Added InstallMissingDrivers because HyperV Core has no built-in Device Manager
'v1.4
'-Added SetKeyboardLayout because HyperV installation should be with English ISO but you might have a different keyboard layout than English (possible password wrong cause)
'v1.3
'-Added question to change password of the built-in Administrator Account
'-Added password never expires guide for administrator accounts
'v1.2
'-Added Standard Folders (folders that will hold the virtual hard disk, vm-infos, iso-images, etc.)
'v1.1
'-Added Partitioning Guide
'v1.0
'-Created Install Script
'
'On Error Resume Next

Set objFSO = CreateObject("Scripting.FileSystemObject")
Set objWSH = CreateObject("WScript.Shell")
Set objNET = CreateObject("WScript.Network")
Set objSHELL = CreateObject("Shell.Application")

Sub Write2Log(ByVal data)
	LogFilePath = Replace(WScript.ScriptFullName, ".vbs", ".log")
	Set objLOG = objFSO.OpenTextFile(LogFilePath, 8, True)
	objLOG.Write data
	objLOG.Close()
End Sub

Sub Write2File(ByVal TargetFile, ByVal content)
	Set objFILE = objFSO.OpenTextFile(TargetFile, 2, True)
	objFILE.Write content
	objFILE.Close()
End Sub

Function CreateNetworkShare(ByVal SourceFolder, ByVal UNCName)
	retBool = True
	Const strComputer = "."
	Const FILE_SHARE = 0
	Const MAXIMUM_CONNECTIONS = 16777216		'Std value for Windows Server 2008R2 Shares
	Set objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2")
	Set objSECDESCR = objWMIService.Get("Win32_SecurityDescriptor").SpawnInstance_()
	Set Trustee = objWMIService.Get("Win32_Trustee").SpawnInstance_
	Trustee.Domain = Null
	Trustee.Name = "EVERYONE"
	Trustee.Properties_.Item("SID") = Array(1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0)
	Set objACE = objWMIService.Get("Win32_Ace").SpawnInstance_
	objACE.Properties_.Item("AccessMask") = 2032127		'2032127 = "Full"; 1245631 = "Change"; 1179817 = "Read"
	objACE.Properties_.Item("AceFlags") = 3
	objACE.Properties_.Item("AceType") = 0
	objACE.Properties_.Item("Trustee") = Trustee
	objSECDESCR.Properties_.Item("DACL") = Array(objACE)
	Set objNewShare = objWMIService.Get("Win32_Share")
	errReturn = objNewShare.Create(SourceFolder, UNCName, FILE_SHARE, MAXIMUM_CONNECTIONS, "", "", objSECDESCR)
	REM objNewShare.Create(_
	REM [Source for share]], _
	REM [Displayed Name of share], _
	REM [0 = It is a normal fileshare], _
	REM [maximum number of concurrent used connections to the share], _
	REM [description for the share], _
	REM [a password that is needed for accessing the share], _
	REM [a security descriptor for the share: Properties > Share > Advanced > Permissions > Everyone: FULL ACCESS])
	If errReturn <> 0 Then retBool = False
	CreateNetworkShare = retBool
End Function

Function RemoveNetworkShare(ByVal UNCName)
	Const strComputer = "."
	Set objSHARES = GetObject("winmgmts:\\" & strComputer & "\root\cimv2").ExecQuery("SELECT * FROM Win32_Share WHERE name = '" & UNCName & "'")
	For Each objSHARE in objSHARES
		MsgBox objSHARE.Name
		objSHARE.Delete()
	Next
End Function

Sub RunWindowsBackupNow()
	objWSH.Run "schtasks /Run /TN " & Chr(34) & "HyperV-Backup" & Chr(34), 1, False
End Sub

Function Convert2TaskTime(ByVal DateTimeValue)
	' Convert a DateTime value to the format needed by 
	' task scheduler
	' YYYY-MM-DDTHH:MM:SS 
	strTime = year(DateTimeValue) & "-"
	strTime = strTime & Right("0" & Month(DateTimeValue), 2) & "-"
	strTime = strTime & Right("0" & Day(DateTimeValue), 2) & "T"
	strTime = strTime & Right("0" & Hour(DateTimeValue), 2) & ":"
	strTime = strTime & Right("0" & Minute(DateTimeValue), 2) & ":"
	strTime = strTime & Right("0" & Second(DateTimeValue), 2)
	Convert2TaskTime = strTime
End Function

Sub CreateDailyScheduledTask(ByVal TaskName, ByVal TaskUser, ByVal TaskUserPwd, ByVal TaskAction)
	Set objSCHEDULE = CreateObject("Schedule.Service")
	Call objSCHEDULE.Connect()
	Set objTASKDEF = objSCHEDULE.NewTask(0)		'0 = running a program
	REM General Tab
	Set objTASKROOT = objSCHEDULE.GetFolder("\")
	Set objTASKINFO = objTASKDEF.RegistrationInfo
	objTASKINFO.Description = ""
	objTASKINFO.Author = TaskUser
	objTASKDEF.Principal.UserId = TaskUser
	objTASKDEF.Principal.LogonType = 6		'6 = Whether user is logged in or not, password needs to be provided
	objTASKDEF.Principal.RunLevel = 0		'0 = User Eval, 1 = UAC Eval
	Set objTASKSET = objTASKDEF.Settings
	objTASKSET.Enabled = True
	REM Triggers Tab
	Set objTRIGGERCOL = objTASKDEF.Triggers
	Set objTRIGGER = objTRIGGERCOL.Create(2)		'2 = Daily Task
	objTRIGGER.Enabled = True
	objTRIGGER.Id = "DailyTriggerID" & "2016"
	objTRIGGER.StartBoundary = Convert2TaskTime(Date - 1 + TimeSerial(3,30,0))
	'objTRIGGER.ExecutionTimeLimit = "P1H"		'Kill task after 1 hour runtime
	objTRIGGER.DaysInterval = 1
	'objTRIGGER.Repetition.Duration = "P1D"		'Repeat for 1 day
	'objTRIGGER.Repetition.Interval = "P1M"		'Repeat task every minute
	REM Action Tab
	Set objTASKACTION = objTASKDEF.Actions.Create(0)		'0 = running a program
	actionPath = TaskAction
	If InStr(1, actionPath, " ", vbTextCompare) > 0 Then
		If Left(actionPath, 1) <> Chr(34) Then actionPath = Chr(34) & actionPath
		If Right(actionPath, 1) <> Chr(34) Then actionPath = actionPath & Chr(34)
	End If
	objTASKACTION.Path = actionPath
	objTASKACTION.WorkingDirectory = Left(TaskAction, InStrRev(TaskAction, "\") - 1)
	REM Create the Task, &H06& = CREATE TASK OR UPDATE TASK, 6 = Whether user is logged in or not, password needs to be provided
	Call objTASKROOT.RegisterTaskDefinition(TaskName, objTASKDEF, &H06&, TaskUser, TaskUserPwd, 6)
End Sub

Sub ShowScheduledTasks()
	Const exportTarget = "C:\Scripts\SchedExp.csv"
	shellstr = "cmd /c schtasks /query /FO CSV /HRESULT /NH /V > " & Chr(34) & exportTarget & Chr(34)
	objWSH.Run shellstr, 1, True
	shellstr2 = "cmd /c notepad.exe " & Chr(34) & exportTarget & Chr(34)
	objWSH.Run shellstr2, 1, True
	objFSO.DeleteFile exportTarget
End Sub

Sub AskForInitialConfigDone()
	selection = MsgBox("Before using this install script make sure you have already done these things:" & vbCrLf & vbCrLf & _
		"- Selected Option 7 from the blue Hyper V sconfig menu (Enabled RDP)" & vbCrLf & _
		"- Selected Option 2 from the blue Hyper V sconfig menu (Set computername)" & vbCrLf & _
		"- Selected Option 8 from the blue Hyper V sconfig menu (Set a static IP address)" & vbCrLf & _
		"- Selected Option 3 from the blue Hyper V sconfig menu (Add local admin)" & vbCrLf & _
		"- Selected Option 9 from the blue Hyper V sconfig menu (Set correct timezone)", vbOkCancel)
	If selection <> vbOK Then WScript.Quit(1)
End Sub

Sub FirewallAllowRDP()
	answer = MsgBox("Do you want to allow RDP-Connections in your windows firewall?", vbYesNo)
	If answer = vbYes Then
		objWSH.Run "cmd /c netsh advfirewall firewall set rule group=" & Chr(34) & "Remote Desktop" & Chr(34) & " new enable=yes", 0, True
	End If
End Sub

Sub CreateDataPartition()
	answer = MsgBox("Do you want to create a ntfs-partition out of the remaining harddisk-space?", vbYesNo)
	If answer = vbYes Then
		objWSH.Run "cmd /c diskpart", 1, False
		MsgBox "A new CMD window running windows DISKPART-Tool just opened." & vbCrLf & _
			"Please type in the following commands there:" & vbCrLf & _
			"list disk" & vbCrLf & _
			"select disk 0" & vbCrLf & _
			"list partition" & vbCrLf & _
			"create partition primary" & vbCrLf & _
			"list partition" & vbCrLf & _
			"select partition 3" & vbCrLf & _
			"format fs=ntfs quick" & vbCrLf & _
			"assign letter = d:" & vbCrLf & _
			"exit", vbExclamation
	End If
End Sub

Sub CreateStandardFolders()
	arrFolders = Array("C:\Installs\", "C:\Scripts\", "D:\iso", "D:\vhd", "D:\vms", "D:\backups", "D:\templates")
	For Each folder In arrFolders
		If Not objFSO.FolderExists(folder) Then
			objFSO.CreateFolder(folder)
		End If
	Next
	Call CreateNetworkShare("D:\backups", "backups")
End Sub

Sub ChangeAdminPassword()
	answer = MsgBox("Do you want to change the password of user Administrator?", vbYesNo)
	If answer = vbYes Then
		newPassword = InputBox("Enter new Password for Administrator:", "New Password")
		If newPassword = "" Then Exit Sub
		objWSH.Run "cmd /c net user Administrator " & newPassword, 0, True
		MsgBox "Password changed successfully!", vbInformation
	End If
End Sub

Sub SetKeyboardLayout()
	Const STD_KEYBOARD_LAYOUT = "de-DE"
	answer = MsgBox("Do you want to change your keyboard layout (keyboard-language)?", vbYesNo)
	If answer = vbYes Then
		keyboardLayout = InputBox("Set your keyboardLayout here like this:" & vbCrLf & "de-DE" & vbCrLf & "en-US" & vbCrLf & "etc...", "Keyboard Layout", STD_KEYBOARD_LAYOUT)
		If keyboardLayout = "" Then Exit Sub
		objWSH.Run "cmd /c powershell.exe -ExecutionPolicy unrestricted -Noninteractive -Command " & _
			Chr(34) & "Set-WinUserLanguageList -LanguageList " & keyboardLayout & Chr(34) & " -Force", 0, True
		answer2 = MsgBox("Your keyboard layout has been set to: " & keyboardLayout & "." & vbCrLf & _
			"Do you want to use this layout for every other user, that will be created in the future?", vbYesNo)
		If answer2 = vbYes Then
			objWSH.Run "cmd /c intl.cpl", 2, False
			MsgBox "A new window just opened. " & vbCrLf & _
				"In this window, select Tab:Administrative > Copy settings... > Here you can set ticks for Copy to " & _
				Chr(34) & "New User Accounts" & Chr(34) & " and " & Chr(34) & "Welcome screen and system accounts" & Chr(34) & vbCrLf & _
				"When you are done, click OK.", vbExclamation
		End If
	End If
End Sub

Sub SetPasswordNeverExpires()
	answer = MsgBox("Do you want to that the passwords of certain accounts never expire?", vbYesNo)
	If answer = vbYes Then
		targetAccount = "Administrator"
		Do
			targetAccount = InputBox("Type in the name of a user that should never expire it's password:" & vbCrLf & _
				"Example:" & vbCrLf & _
				"Administrator", "Target Accountname", targetAccount)
			If targetAccount = "" Then Exit Sub
			objWSH.Run "cmd /c wmic USERACCOUNT WHERE " & Chr(34) & "Name='" & targetAccount & "'" & Chr(34) & " set PasswordExpires=FALSE", 0, True
			MsgBox "Successfully set password expire=never on account: " & targetAccount, vbInformation
			exitAnswer = MsgBox("Do you want to set password expire=never for another account?", vbYesNo)
		Loop Until (exitAnswer = vbNo)
	End If
End Sub

Sub InstallMissingDrivers()
	answer = MsgBox("Do you want to install missing drivers now?" & vbCrLf & _
		"(Yes will open a notepad. Close it when you're done)", vbYesNo)
	If answer = vbYes Then
		output = ""
		strComputer = "."
		Set objWMIService = GetObject(_
			"winmgmts:\\" & strComputer & "\root\cimv2")
		Set colItems = objWMIService.ExecQuery("Select * from Win32_PnPEntity WHERE ConfigManagerErrorCode <> 0")
		For Each objItem in colItems
			output = output & _
			"Class GUID: " & objItem.ClassGuid & vbCrLf & _
			"Description: " & objItem.Description & vbCrLf & _
			"Device ID: " & objItem.DeviceID & vbCrLf & _
			"Manufacturer: " & objItem.Manufacturer & vbCrLf & _
			"Name: " & objItem.Name & vbCrLf & _
			"PNP Device ID: " & objItem.PNPDeviceID & vbCrLf & _
			"Service: " & objItem.Service & vbCrLf & vbCrLf
		Next
		Call Write2File("C:\Scripts\missing-drivers.log", output)
		WScript.Sleep 500
		objWSH.Run "cmd /c notepad.exe C:\Scripts\missing-drivers.log", 0, True
	End If
End Sub

Sub AddWindowsBackupRole()
	answer = MsgBox("Do you want to install the Windows-Server-Backup Role on this server?", vbYesNo)
	If answer = vbYes Then
		objWSH.Run "cmd /c powershell.exe -ExecutionPolicy unrestricted -Noninteractive -Command " & _
			Chr(34) & "Add-WindowsFeature -Name Windows-Server-Backup", 1, True
		MsgBox "Windows-Server-Backup Role was successfully installed!", vbInformation
	End If
End Sub

Sub InstallHVMTools()
	Const HKEY_LOCAL_MACHINE = &h80000002
	answer = MsgBox("Do you want to install the free HVMTools for manageing Hyper V?", vbYesNo)
	If answer = vbYes Then
		doneAnswer = InputBox("Download the free HVMTools from this URL and install it. Click OK when you are ready", "HVMTools Install", "http://www.probus-it.se/hypervtools/hypervmanager")
		If doneAnswer = "" Then Exit Sub
		Call Write2File("C:\Scripts\HVMStartup.cmd", "@echo off" & vbCrLf & "start hvm" & vbCrLf)
		WScript.Sleep 500
		Set objREG = GetObject("winmgmts:{impersonationLevel=impersonate}!\\" & "." & "\root\default:StdRegProv")
		objREG.SetStringValue HKEY_LOCAL_MACHINE, "SOFTWARE\Microsoft\Windows\CurrentVersion\run", "HVM", "C:\Scripts\HVMStartup.cmd"
	End If
End Sub

Sub ConfigureDailyBackup()
	answer = MsgBox("Do you want to install a scheduled task," & vbCrLf & _
		"that will backup your machine once per day?", vbYesNo)
	If answer = vbYes Then
		user4Task = InputBox("Enter a user that will run the task", "Task User", "Administrator")
		If user4Task = "" Then Exit Sub
		pwd4Task = InputBox("Enter a the password of that user", "Task Userpassword", "secret123")
		If pwd4Task = "" Then Exit Sub
		Const vhdFolder = "D:\vhd"
		strServers = ""		'"srv01de,testsrv03de,testsrv04de"
		If objFSO.FolderExists(vhdFolder) Then
			Set objFOLDER = objFSO.GetFolder(vhdFolder)
			For Each file In objFOLDER.Files
				fileExt = "." & objFSO.GetExtensionName(file.Name)
				fileNameNoExt = Replace(file.Name, fileExt, "")
				strServers = strServers & fileNameNoExt & ","
			Next
			If strServers <> "" Then
				strServers = Left(strServers, Len(strServers) - 1)
			End If
		End If
		If strServers = "" Then strServers = "srv01de"
		REM Use this section for backup with daily history (each day will backup to a different subfolder, old backups need to be deleted manually)
		content = _
			"Set objWSH = CreateObject(" & Chr(34) & "WScript.Shell" & Chr(34) & ")" & vbCrLf & _
			"Set objFSO = CreateObject(" & Chr(34) & "Scripting.FileSystemObject" & Chr(34) & ")" & vbCrLf & _
			"currentYear = Datepart(" & Chr(34) & "yyyy" & Chr(34) & ", Now)" & vbCrLf & _
			"currentMonth = Right(" & Chr(34) & "0" & Chr(34) & " & Month(Now), 2)" & vbCrLf & _
			"currentDay = Right(" & Chr(34) & "0" & Chr(34) & " & Day(Now), 2)" & vbCrLf & _
			"dateFolder = currentDay & " & Chr(34) & "-" & Chr(34) & " & currentMonth & " & Chr(34) & "-" & Chr(34) & " & currentYear" & vbCrLf & _
			"dateFolder = " & Chr(34) & "\\localhost\backups\" & Chr(34) & " & dateFolder" & vbCrLf & _
			"If Not objFSO.FolderExists(dateFolder) Then objFSO.CreateFolder dateFolder" & vbCrLf & _
			"shellstr = " & Chr(34) & "wbadmin.exe start backup" & _
				" -backupTarget:" & Chr(34) & " & dateFolder & " & Chr(34) & _
				" -hyperv:" & Chr(34) & " & Chr(34) & " & Chr(34) & strServers & Chr(34) & " & Chr(34) & " & Chr(34) & _
				" -allcritical -vssfull -quiet" & Chr(34) & vbCrLf & _
			"objWSH.Run shellstr, 1, True"
		Call Write2File("C:\Scripts\hyperv-backup.vbs", content)
		Call CreateDailyScheduledTask("HyperV-Backup", user4Task, pwd4Task, "C:\Scripts\hyperv-backup.vbs")
		REM Use this section for incrementell-forever backup (Backup will be overwritten everyday)
		'shellstr = "wbadmin.exe start backup -backupTarget:\\localhost\backups -hyperv:" & Chr(34) & strServers & Chr(34) & " -allcritical -vssfull -quiet"
		'Call Write2File("C:\Scripts\hyperv-backup.cmd", shellstr)
		'Call CreateDailyScheduledTask("HyperV-Backup", user4Task, pwd4Task, "C:\Scripts\hyperv-backup.cmd")
	End If
End Sub

Sub CopyBackupSecure(ByVal SourceFolder, ByVal TargetFolder)
	answer = MsgBox("Do you want to copy the folder " & vbCrLf & _
		Chr(34) & SourceFolder & Chr(34) & " to target " & Chr(34) & TargetFolder & Chr(34) & "?", vbYesNo)
	If answer = vbYes Then
		shellstr = Chr(34) & "C:\Windows\system32\robocopy.exe" & Chr(34) & " " & _
			Chr(34) & SourceFolder & Chr(34) & " " & _
			Chr(34) & TargetFolder & Chr(34) & " " & _
			"/MIR /COPYALL /B /R:2 /Log:" & _
			Chr(34) & "C:\Scripts\copylog.txt" & Chr(34)
		objWSH.Run shellstr, 1, True
	End If
End Sub

Sub Main()
	Call AskForInitialConfigDone()
	Call FirewallAllowRDP()
	Call CreateDataPartition()
	Call CreateStandardFolders()
	Call ChangeAdminPassword()
	Call SetKeyboardLayout()
	Call SetPasswordNeverExpires()
	Call InstallMissingDrivers()
	Call AddWindowsBackupRole()
	Call InstallHVMTools()
	Call ConfigureDailyBackup()
	REM Examples you can run manually
	'Call RemoveNetworkShare("backups")
	'Call RunWindowsBackupNow()
	'Call CopyBackupSecure("D:\backups\WindowsImageBackup", "F:\")
	'Call ShowScheduledTasks()
	MsgBox "Script ended successfully. No Errors occured", vbInformation
	WScript.Quit(0)
End Sub

If Err.Number = 0 Then Main()
